@@ -1,9 +1,9 @@
 import base64
+import os
+import thread
 from sqlobject.dbconnection import DBAPI
-from sqlobject.col import popKey
 from sqlobject import col, sqlbuilder
 from sqlobject.dberrors import *
-import thread
 
 sqlite = None
 using_sqlite2 = False
@@ -28,11 +28,11 @@ class SQLiteConnection(DBAPI):
         global using_sqlite2
         if sqlite is None:
             try:
-                import sqlite3 as sqlite
+                from pysqlite2 import dbapi2 as sqlite
                 using_sqlite2 = True
             except ImportError:
                 try:
-                    from pysqlite2 import dbapi2 as sqlite
+                    import sqlite3 as sqlite
                     using_sqlite2 = True
                 except ImportError:
                     import sqlite
@@ -64,27 +64,27 @@ class SQLiteConnection(DBAPI):
                 sqlite2_Binary = sqlite.Binary
                 sqlite.Binary = lambda s: sqlite2_Binary(sqlite.encode(s))
             if 'factory' in kw:
-                factory = popKey(kw, 'factory')
+                factory = kw.pop('factory')
                 if isinstance(factory, str):
                     factory = globals()[factory]
                 opts['factory'] = factory(sqlite)
         else:
             opts['autocommit'] = bool(autoCommit)
             if 'encoding' in kw:
-                opts['encoding'] = popKey(kw, 'encoding')
+                opts['encoding'] = kw.pop('encoding')
             if 'mode' in kw:
-                opts['mode'] = int(popKey(kw, 'mode'), 0)
+                opts['mode'] = int(kw.pop('mode'), 0)
         if 'timeout' in kw:
             if using_sqlite2:
-                opts['timeout'] = float(popKey(kw, 'timeout'))
+                opts['timeout'] = float(kw.pop('timeout'))
             else:
-                opts['timeout'] = int(float(popKey(kw, 'timeout')) * 1000)
+                opts['timeout'] = int(float(kw.pop('timeout')) * 1000)
         if 'check_same_thread' in kw:
-            opts["check_same_thread"] = bool(popKey(kw, 'check_same_thread'))
+            opts["check_same_thread"] = bool(kw.pop('check_same_thread'))
         # use only one connection for sqlite - supports multiple)
         # cursors per connection
         self._connOptions = opts
-        self.use_table_info = popKey(kw, "use_table_info", False)
+        self.use_table_info = kw.pop("use_table_info", False)
         DBAPI.__init__(self, **kw)
         self._threadPool = {}
         self._threadOrigination = {}
@@ -106,7 +106,12 @@ class SQLiteConnection(DBAPI):
     connectionFromURI = classmethod(connectionFromURI)
 
     def uri(self):
-        return 'sqlite:///%s' % self.filename
+        path = self.filename
+        if path == ":memory:":
+            path = "/:memory:"
+        else:
+            path = "//" + path
+        return 'sqlite:%s' % path
 
     def getConnection(self):
         # SQLite can't share connections between threads, and so can't
@@ -226,12 +231,13 @@ class SQLiteConnection(DBAPI):
         else:
             return DBAPI._insertSQL(self, table, names, values)
 
-    def _queryAddLimitOffset(self, query, start, end):
+    def _queryAddLimitOffset(cls, query, start, end):
         if not start:
             return "%s LIMIT %i" % (query, end)
         if not end:
             return "%s LIMIT 0 OFFSET %i" % (query, start)
         return "%s LIMIT %i OFFSET %i" % (query, end-start, start)
+    _queryAddLimitOffset = classmethod(_queryAddLimitOffset)
 
     def createColumn(self, soClass, col):
         return col.sqliteCreateSQL()
@@ -274,7 +280,7 @@ class SQLiteConnection(DBAPI):
                         for col in sqlmeta.columnList if col.name != column.name]
         cols = ",\n".join(["    %s" % c for c in cols])
         self.query('CREATE TABLE %s (\n%s\n)' % (sqlmeta.table, cols))
-        all_columns = ', '.join(['id'] + [col.dbName for col in sqlmeta.columnList])
+        all_columns = ', '.join([sqlmeta.idName] + [col.dbName for col in sqlmeta.columnList])
         self.query('INSERT INTO %s (%s) SELECT %s FROM %s' % (
             sqlmeta.table, all_columns, all_columns, new_name))
         self.query('DROP TABLE %s' % new_name)
@@ -289,9 +295,12 @@ class SQLiteConnection(DBAPI):
         colData = self.queryAll("PRAGMA table_info(%s)" % tableName)
         results = []
         for index, field, t, nullAllowed, default, key in colData:
-            if field == 'id':
+            if field == soClass.sqlmeta.idName:
                 continue
             colClass, kw = self.guessClass(t)
+            if default == 'NULL':
+                nullAllowed = True
+                default = None
             kw['name'] = soClass.sqlmeta.style.dbColumnToPythonAttr(field)
             kw['dbName'] = field
             kw['notNone'] = not nullAllowed
@@ -305,12 +314,14 @@ class SQLiteConnection(DBAPI):
         colData = self.queryOne("SELECT sql FROM sqlite_master WHERE type='table' AND name='%s'"
                                 % tableName)
         if not colData:
-            raise ValueError('The table %s ws not found in the database. Load failed.' % tableName)
+            raise ValueError('The table %s was not found in the database. Load failed.' % tableName)
         colData = colData[0].split('(', 1)[1].strip()[:-2]
-        while colData.find('(') > -1:
-            st = colData.find('(')
-            en = colData.find(')')
-            colData = colData[:st] + colData[en+1:]
+        while True:
+            start = colData.find('(')
+            if start == -1: break
+            end = colData.find(')', start)
+            if end == -1: break
+            colData = colData[:start] + colData[end+1:]
         results = []
         for colDesc in colData.split(','):
             parts = colDesc.strip().split(' ', 2)
@@ -352,8 +363,21 @@ class SQLiteConnection(DBAPI):
             return col.FloatCol, {}
         elif t.find('DECIMAL') >= 0:
             return col.DecimalCol, {}
+        elif t.find('BOOL') >= 0:
+            return col.BoolCol, {}
         else:
             return col.Col, {}
 
+    def createEmptyDatabase(self):
+        if self._memory:
+            return
+        open(self.filename, 'w').close()
+
+    def dropDatabase(self):
+        if self._memory:
+            return
+        os.unlink(self.filename)
+
+
 def stop_pysqlite2_converting_strings(s):
     return s
